<?php

namespace Drupal\snippet_manager;

use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Component\Utility\Timer;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityViewBuilderInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * View builder for snippets.
 */
class SnippetViewBuilder implements EntityViewBuilderInterface, EntityHandlerInterface {

  use StringTranslationTrait;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Snippet variable manager.
   *
   * @var \Drupal\snippet_manager\SnippetVariablePluginManager
   */
  protected $variableManager;

  /**
   * Constructs a new SnippetViewBuilder.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\snippet_manager\SnippetVariablePluginManager $variable_manager
   *   Snippet variable manager.
   */
  public function __construct(ModuleHandlerInterface $module_handler, SnippetVariablePluginManager $variable_manager) {
    $this->moduleHandler = $module_handler;
    $this->variableManager = $variable_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $container->get('module_handler'),
      $container->get('plugin.manager.snippet_variable')
    );
  }

  /**
   * Builds the render array for the provided snippet.
   *
   * @param \Drupal\Core\Entity\EntityInterface $snippet
   *   The snippet to render.
   * @param string $view_mode
   *   (optional) The view mode that should be used to render the snippet.
   * @param string $langcode
   *   (optional) For which language the entity should be rendered.
   * @param array $context
   *   (optional) Twig context.
   *
   * @return array
   *   A render array for the snippet.
   */
  public function view(EntityInterface $snippet, $view_mode = 'full', $langcode = NULL, array $context = []) {

    /** @var \Drupal\snippet_manager\SnippetInterface $snippet */
    $build = $view_mode == 'source'
      ? $this->viewSource($snippet)
      : $this->viewDefault($snippet, $context);

    $build['snippet']['#cache'] = [
      'keys' => ['entity_view', 'snippet', $snippet->id(), $view_mode],
      'tags' => Cache::mergeTags($this->getCacheTags(), $snippet->getCacheTags()),
    ];

    $this->moduleHandler->alter(['snippet_view'], $build, $snippet, $view_mode);

    return $build;
  }

  /**
   * Builds the render array for the source view mode.
   *
   * @param \Drupal\snippet_manager\SnippetInterface $snippet
   *   The snippet to render.
   *
   * @return array
   *   Render array to display HTML source code of the snippet.
   */
  protected function viewSource(SnippetInterface $snippet) {

    $build['content']['#type'] = 'textarea';

    Timer::start('snippet');

    $value = $this->view($snippet);
    // Render the snippet immediately to calculate render time.
    $build['content']['#value'] = (string) render($value);

    $render_time = Timer::read('snippet');

    $build['content']['#attributes']['class'][] = 'snippet-html-source';

    $build['render_time_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => 'snippet-render-time'],
    ];

    $build['render_time_wrapper']['render_time'] = [
      '#markup' => $this->t('Render time: %time ms', ['%time' => $render_time]),
    ];

    $build['#attached']['library'][] = 'snippet_manager/editor';

    return $build;
  }

  /**
   * Builds render array.
   *
   * @param \Drupal\snippet_manager\SnippetInterface $snippet
   *   A snippet.
   * @param array $extra_context
   *   (optional) Twig context.
   *
   * @return array
   *   Snippet content as render array.
   */
  protected function viewDefault(SnippetInterface $snippet, array $extra_context = []) {

    $context = static::getDefaultContext();

    // @todo Remove this condition once we drop support for Drupal 8.4.
    if (method_exists($this->moduleHandler, 'invokeAllDeprecated')) {
      $deprecation_message = 'hook_snippet_context has been deprecated';
      // The context is an array of processed Twig variables.
      $context += $this->moduleHandler->invokeAllDeprecated($deprecation_message, 'snippet_context', [$snippet]);
    }
    else {
      $context += $this->moduleHandler->invokeAll('snippet_context', [$snippet]);
    }

    /** @var \Drupal\snippet_manager\SnippetVariablePluginManager $variable_manager */
    foreach ($snippet->get('variables') as $variable_name => $variable) {
      try {
        $plugin = $this->variableManager->createInstance(
          $variable['plugin_id'],
          $variable['configuration']
        );
        $context[$variable_name] = $plugin->build();
      }
      catch (PluginNotFoundException $exception) {
        $context[$variable_name] = '';
      }
    }

    // Extra context should take precedence.
    $context = $extra_context + $context;

    // @todo Remove this condition once we drop support for Drupal 8.4.
    if (method_exists($this->moduleHandler, 'alterDeprecated')) {
      $deprecation_message = 'hook_snippet_context_alter has been deprecated';
      $this->moduleHandler->alterDeprecated($deprecation_message, 'snippet_context', $context, $snippet);
    }
    else {
      $this->moduleHandler->alter('snippet_context', $context, $snippet);
    }

    $template = $snippet->get('template');

    $build['snippet'] = [
      '#type' => 'inline_template',
      '#template' => check_markup($template['value'], $template['format']),
      '#context' => $context,
    ];

    if ($snippet->get('css')['status'] || $snippet->get('js')['status']) {
      $build['snippet']['#attached']['library'][] = 'snippet_manager/snippet_' . $snippet->id();
    }

    return $build;
  }

  /**
   * Returns default Twig context.
   *
   * @return array
   *   Twig context.
   */
  protected static function getDefaultContext() {

    $context = [];

    $theme = \Drupal::theme()->getActiveTheme();
    $context['theme'] = $theme->getName();
    $context['theme_directory'] = $theme->getPath();

    $context['base_path'] = base_path();
    $context['front_page'] = Url::fromRoute('<front>');
    $context['is_front'] = \Drupal::service('path.matcher')->isFrontPage();

    $user = \Drupal::currentUser();
    $context['is_admin'] = $user->hasPermission('access administration pages');
    $context['logged_in'] = $user->isAuthenticated();

    return $context;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    return ['snippet_view'];
  }

  /**
   * {@inheritdoc}
   */
  public function resetCache(array $entities = NULL) {
    // @see \Drupal\Core\Entity\EntityViewBuilder::resetCache()
    if (isset($entities)) {
      $tags = [];
      foreach ($entities as $entity) {
        $tags = Cache::mergeTags($tags, $entity->getCacheTags());
        $tags = Cache::mergeTags($tags, $entity->getEntityType()->getListCacheTags());
      }
      Cache::invalidateTags($tags);
    }
    else {
      Cache::invalidateTags($this->getCacheTags());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function viewMultiple(array $entities = [], $view_mode = 'full', $langcode = NULL) {
    throw new \LogicException();
  }

  /**
   * {@inheritdoc}
   */
  public function viewFieldItem(FieldItemInterface $item, $display_options = []) {
    throw new \LogicException();
  }

  /**
   * {@inheritdoc}
   */
  public function viewField(FieldItemListInterface $items, $display_options = []) {
    throw new \LogicException();
  }

  /**
   * {@inheritdoc}
   */
  public function buildComponents(array &$build, array $entities, array $displays, $view_mode) {
    throw new \LogicException();
  }

}
